from configparser import ConfigParser
import logging
from pprint import pformat

import tator

from hms_import.util import validate_type_from_config
from hms_import.o2.import_videos import import_videos
from hms_import.o2.import_metadata import import_metadata
from hms_import.summary_image import create_summary_image


logger = logging.getLogger(__name__)


def main(config_file: str):
    # Read and parse the config file
    logger.info("Parsing config file %s", config_file)
    config = ConfigParser()
    config.read(config_file)

    # Validate config values
    host = config["Tator"]["Host"]
    token = config["Tator"]["Token"]
    project_id = config["Tator"].getint("ProjectId", -1)

    tator_api = tator.get_api(host=host, token=token)
    if project_id < 0:
        raise ValueError(f"Missing ProjectId value in the Tator section of the config file")
    try:
        tator_api.get_project(project_id)
    except Exception as exc:
        raise ValueError(f"Could not get project {project_id}") from exc

    file_type = validate_type_from_config(tator_api, config, "FileType", project_id, "file_type")
    media_type = validate_type_from_config(
        tator_api, config, "MediaType", project_id, "media_type", list_filters=[("dtype", "video")]
    )
    summary_type = validate_type_from_config(
        tator_api,
        config,
        "SummaryType",
        project_id,
        "media_type",
        list_filters=[("dtype", "image")],
    )
    directory = config["Local"]["Directory"]

    # Create trip summary image
    create_summary_kwargs = {
        "host": host,
        "media_type": summary_type,
        "directory": directory,
        "import_type": "O2",
        "hdd_sn": config["Trip"].get("HddSerialNumber", None),
    }
    logger.info("Creating trip summary with configuration %s", pformat(create_summary_kwargs))
    section, shared_attrs = create_summary_image(token=token, **create_summary_kwargs)
    if section:
        logger.info("Created trip summary in section %s", section)
    else:
        raise RuntimeError("Could not create trip summary and section")

    # Construct metadata argument dictionary for logging (do not log token for security)
    import_metadata_kwargs = {
        "host": host,
        "project_id": project_id,
        "directory": directory,
        "meta_ext": config["Local"]["MetadataExtension"],
        "file_type": file_type,
    }
    logger.info("Starting metadata import with configuration %s", pformat(import_metadata_kwargs))
    file_ids = import_metadata(token=token, **import_metadata_kwargs)
    logger.info("Metadata import complete!")

    # Construct video argument dictionary for logging (do not log token for security)
    import_video_kwargs = {
        "host": host,
        "directory": directory,
        "project_id": project_id,
        "file_ids": file_ids,
        "media_ext": config["Local"]["MediaExtension"],
        "media_type": media_type,
        "section": section,
        "shared_attrs": shared_attrs,
    }
    logger.info("Starting video import with configuration %s", pformat(import_video_kwargs))
    media_ids = import_videos(token=token, **import_video_kwargs)
    logger.info("Video import complete!")
    logger.info("Created the following media: %s", pformat(media_ids))

    logger.info(
        "Launching decryption workflow for %d media objects and %d metadata files",
        len(media_ids),
        len(file_ids),
    )

    # Launch one workflow for all media ids
    algorithm_name = config["Tator"]["AlgorithmName"]
    job_spec = {"algorithm_name": algorithm_name, "media_ids": media_ids}
    try:
        response = tator_api.create_job_list(project=project_id, job_spec=job_spec)
    except Exception:
        logger.error(
            "Could not launch job with job_spec=%s in project %d",
            pformat(job_spec),
            project_id,
            exc_info=True,
        )
    else:
        logger.info(
            "Launched workflow %s on media %s (received response %s)",
            algorithm_name,
            pformat(media_ids),
            pformat(response),
        )



import logging
import os

import tator

logger = logging.getLogger(__name__)

def main(
    host: str,
    token: str,
    log_file_type_id: int,
    log_filename: str,
) -> bool:
    """
    :host: The Tator domain
    :token: The REST API token
    :log_file_type_id: The unique ID of the type of file to create for storing GPS data
    :log_filename: The name of an existing log file to upload to Tator
    :returns: True if the import was successful, False if any part of it fails
    """
    tator_api = tator.get_api(host=host, token=token)
    r = None
    try:
        for p, r in tator.util.upload_generic_file(api=tator_api, file_type=log_file_type_id, path=log_filename, description="Log file from hms-import", name=os.path.basename(log_filename), timeout=120):
            logger.info("Upload progress: %.1f%%", p)
    except Exception:
        logger.error("Exception while uploading log file", exc_info=True)

    if r:
        logger.info(r)
        return True
    return False



from datetime import datetime
from glob import iglob
import logging
import os
import re
from tempfile import NamedTemporaryFile
from typing import Optional, Tuple

from PIL import Image, ImageDraw, ImageFont
import tator
from tator.openapi.tator_openapi import CreateResponse

from hms_import.util import VESSEL_NAME_MAP, build_section_name


logger = logging.getLogger(__name__)
FONT = os.path.join(os.path.dirname(__file__), "Cascadia.ttf")
FONT_SIZE = 36
SUMMARY_IMG_SZ = 1024


def create_summary_image(
    host: str,
    token: str,
    media_type: int,
    directory: str,
    import_type: str,
    hdd_sn: Optional[str] = None,
) -> Tuple[str, str]:
    """Creates a summary image for a trip.

    :host: The Tator hostname
    :token: The Tator API token
    :media_type: The Tator media type to create
    :directory: The directory containing files to use to determine the vessel name
    :hdd_sn: The value for the HDD Serial Number attribute
    :returns: The created section name and found vessel name
    """

    vessel_name = "UNKNOWN"
    if import_type == "O2":
        # Get the vessel name and trip id from the first filename found starting with `stream-`
        filename = os.path.basename(next(iglob(os.path.join(directory, f"stream-*"))))
        vessel_trip_match = re.match(r"^stream-([^-]*)-.*$", filename)
        if vessel_trip_match:
            try:
                vessel_name = vessel_trip_match.group(1)
            except Exception:
                logger.warning("Could not find vessel and trip name in '%s'", filename)
    elif import_type == "B3":
        video_list = list(iglob(os.path.join(directory, f"*.mp4")))
        video_path = video_list[0]
        filename = os.path.basename(video_path)
        filename_parts = os.path.splitext(filename)[0].split("-")
        if len(filename_parts) == 4:
            try:
                vessel_name = filename_parts[0]
            except Exception:
                logger.warning("Could not get the vessel or section name", exc_info=True)
    else:
        raise RuntimeError(f"Received unexepected import type argument: {import_type}")

    vessel_name = VESSEL_NAME_MAP.get(vessel_name, vessel_name)
    attrs = {"Vessel Name": vessel_name}
    section_name = build_section_name(vessel_name)
    attrs = {
        "Vessel Name": vessel_name,
        "HDD Date Received": datetime.today().strftime("%Y-%m-%d"),
        "HDD Serial Number": hdd_sn,
    }
    label_len = max(len(key) for key in attrs.keys())
    image_text = "\n".join(f"{k: >{label_len}}: {v}" for k, v in attrs.items())
    font = ImageFont.truetype(font=FONT, size=FONT_SIZE)
    image = Image.new("RGB", (SUMMARY_IMG_SZ, SUMMARY_IMG_SZ), color="black")
    draw = ImageDraw.Draw(image)
    left, top, right, bottom = draw.textbbox((0, 0), image_text, font=font)
    x_pos = (SUMMARY_IMG_SZ - (right - left)) // 2
    y_pos = (SUMMARY_IMG_SZ - (bottom - top)) // 2
    draw.text((x_pos, y_pos), image_text, fill="white", font=font)

    with NamedTemporaryFile(suffix=".png") as temp_file:
        image.save(temp_file.name)
        tator_api = tator.get_api(host=host, token=token)

        import_media_args = {
            "api": tator_api,
            "type_id": media_type,
            "path": temp_file.name,
            "section": section_name,
            "fname": os.path.basename(temp_file.name),
            "attributes": {k: v for k, v in attrs.items() if v is not None},
        }

        response = None
        try:
            for progress, response in tator.util.upload_media(**import_media_args):
                logger.info("Upload progress for %s: %d%%", temp_file.name, progress)
        except Exception:
            logger.error(
                "Could not create trip summary with args:\n%s", import_media_args, exc_info=True
            )
        else:
            if isinstance(response, CreateResponse):
                logger.info("Uploaded image, received response: %s", response.message)

    return section_name, vessel_name



#!/usr/bin/env python3

from glob import glob
import hashlib
import logging
import os
from pprint import pformat
from typing import Dict, List

import tator
from tator.openapi.tator_openapi.models import CreateListResponse, UploadInfo
from tator.util._upload_file import _upload_file
from tqdm import tqdm


logger = logging.getLogger(__name__)


def _calculate_md5(file_path):
    hash_md5 = hashlib.md5()

    with open(file_path, "rb") as f:
        for chunk in iter(lambda: f.read(4096), b""):
            hash_md5.update(chunk)

    return hash_md5.hexdigest()


def _import_video(
    host: str,
    token: str,
    path: str,
    project_id: int,
    media_type: int,
    file_ids: List[int],
    section: str,
    shared_attrs: Dict[str, str],
) -> int:
    tator_api = tator.get_api(host=host, token=token)

    # Perform idempotency check
    filename = os.path.basename(path)
    paginator = tator.util.get_paginator(tator_api, "get_media_list")
    page_iter = paginator.paginate(project=project_id, name=filename, type=media_type)
    try:
        media = next(media for page in page_iter for media in page)
    except (RuntimeError, StopIteration):
        # Bug in paginator will raise RuntimeError if zero entities are returned
        pass
    else:
        logger.info("Found existing media %d with same file name, skipping upload", media.id)
        return media.id

    # Upload encrypted file to Tator
    logger.info("Uploading %s", filename)
    response = None
    try:
        for progress, response in _upload_file(tator_api, project_id, path=path, filename=filename):
            logger.info("Upload progress: %.1f%%", progress)
    except Exception as exc:
        raise RuntimeError(f"Raised exception while uploading '{filename}', skipping") from exc

    if not isinstance(response, UploadInfo) or response.key is None:
        raise RuntimeError(f"Did not upload '{filename}', skipping")

    logger.info("Uploading %s successful!", filename)

    # Create a media object containing the key to the uploaded file
    media_spec = {
        "type": media_type,
        "section": section,
        "name": filename,
        "md5": _calculate_md5(path),
        "attributes": {
            **shared_attrs,
            "encrypted_path": response.key,
            "related_files": ",".join(str(file_id) for file_id in file_ids),
        },
    }

    logger.info(
        "Creating media object in project %d with media_spec=%s", project_id, pformat(media_spec)
    )
    try:
        response = tator_api.create_media_list(project_id, [media_spec])
    except Exception as exc:
        raise RuntimeError(f"Could not create media with {project_id=} and {media_spec=}") from exc

    return response.id[0] if isinstance(response, CreateListResponse) and response.id else 0


def import_videos(
    *,
    host: str,
    token: str,
    directory: str,
    media_ext: str,
    project_id: int,
    media_type: int,
    section: str,
    file_ids: List[int],
    shared_attrs: Dict[str, str],
) -> List[int]:
    """
    Finds all encrypted video files in `directory`, uploads them to Tator, and creates a media
    object referencing the upload. A future algorithm will be responsible for decrypting and
    transcoding them. Disallows use of positional arguments.

    :param host: The hostname of the Tator deployment to upload to.
    :type host: str
    :param token: The Tator API token to use for authentication.
    :type token: str
    :param directory: The directory to search for encrypted video files.
    :type directory: str
    :param media_ext: The extension of the encrypted video files.
    :type media_ext: str
    :param project_id: The integer id of the project to upload the videos to.
    :type project_id: int
    :param media_type: The integer id of the media type to create.
    :type media_type: int
    :param section: The section to import the media to.
    :type section: str
    :param file_ids: The list of file ids to associate imported media with.
    :type file_ids: List[int]
    :param shared_attrs: The dict of shared attributes for all media.
    :type shared_attrs: Dict[str, str]
    """
    file_list = glob(os.path.join(directory, f"*{media_ext}-[0-9]*"))
    logger.info("Found the following files:\n* %s", "\n* ".join(file_list))

    results = []
    for filename in tqdm(
        file_list,
        total=len(file_list),
        desc="Video Imports",
        dynamic_ncols=True,
        position=0,
        ascii=False,
    ):
        try:
            media_id = _import_video(
                host, token, filename, project_id, media_type, file_ids, section, shared_attrs
            )
        except Exception:
            logger.error("Failed to import '%s'", os.path.basename(filename), exc_info=True)
        else:
            if media_id:
                results.append(media_id)
    return results



import argparse
from configparser import ConfigParser
from datetime import date
import logging
import os
from typing import Any, Callable, Optional

from tator.openapi.tator_openapi import Section, TatorApi

logger = logging.getLogger(__name__)

SAIL_DATE_PLACEHOLDER = "NO_SAIL_DATE"
LAND_DATE_PLACEHOLDER = "NO_LAND_DATE"
VESSEL_NAME_MAP = {
    "1022773": "Eagle Eye II",
    "1027010": "Alex Marie",
    "1029712": "Kenny Boy",
    "1031845": "Destiny",
    "1038712": "Ellen Jean",
    "1049623": "Leo B",
    "1061277": "Miss Kaleigh",
    "1075516": "Jamie B",
    "1081640": "Capt Easy",
    "1097631": "Hanna Katherine",
    "1231179": "Full Timer",
    "1231553": "Dayboat One",
    "1275363": "Market Price",
    "507646": "Daytona",
    "510728": "Miss Suzanne",
    "539456": "Bookie",
    "541694": "Eaglet II",
    "557860": "Independence",
    "579208": "Galilean",
    "591301": "Charleston Star",
    "592269": "Capt. Gorman III",
    "595941": "Blue Wave",
    "596805": "Mary Ann",
    "597797": "Bluefin",
    "600048": "Kim Thanh II",
    "600293": "4 Cs",
    "600322": "Bobalou",
    "604915": "Frances Ann",
    "608436": "Christopher Joe",
    "609121": "Carol Ann",
    "610006": "Captain Michael",
    "610080": "Blessings",
    "611243": "Erica Lynn",
    "616696": "Cynthia Renee",
    "620696": "Albi",
    "622027": "Carter Anthony",
    "624406": "Dayboat Too",
    "628300": "Bigeye",
    "630306": "Defiance",
    "633432": "Kelly Ann",
    "638863": "Miss Shannon",
    "641486": "Miss Ann",
    "646506": "Janice Ann",
    "651698": "Intrepido II",
    "652211": "Lisa Ann",
    "655108": "Miss Haylee II",
    "656259": "Kristin Lee",
    "659026": "Eyelander",
    "661098": "Miss Rita",
    "661284": "Hannah Boden",
    "662409": "Fine Tuna",
    "665962": "Linnea C",
    "668316": "Capt. Luu",
    "669872": "Eagle Eye",
    "671174": "Delphinus",
    "673556": "Candace",
    "674417": "Bear",
    "680845": "Endeavor",
    "683311": "Iron Maiden",
    "904300": "Kim Nhi",
    "905679": "Capt. Mike",
    "908045": "Julianne",
    "911431": "Capt. David",
    "912738": "Joshua Nicole",
    "913385": "2nd Wind",
    "918933": "Miss Lena",
    "919799": "Blue Sea I",
    "929813": "Capt. Bob",
    "936041": "Cecelia II",
    "937416": "Kim Thanh",
    "956008": "Dakota",
    "957958": "Miss Jane",
    "969495": "Hannah Story",
    "983321": "Blue Sea 2",
    "997091": "Alexandria Dawn",
    "FL5295PN": "Big B",
    "MD9128BD": "Integrity",
    "MS1993DD": "Angel's Grace",
    "NC2015WS": "Kaitlyn C",
    "NC3377EM": "Big Dave",
    "NC5126DC": "Hardway",
    "NC5516DF": "Fish Wish",
    "NC6362EX": "Big Skipper",
    "NC8831WR": "Body Count",
    "NJ4755FP": "Backwash",
}


def validate_type_from_config(
    api: TatorApi,
    config: ConfigParser,
    config_field: str,
    project_id: int,
    tator_type: str,
    list_filters=None,
) -> int:
    if tator_type not in ["media_type", "file_type"]:
        raise ValueError(f"Cannot validate {config_field=} for {tator_type=}")

    type_id = config["Tator"].getint(config_field, -1)
    if type_id > 0:
        try:
            getattr(api, f"get_{tator_type}")(type_id)
        except Exception as exc:
            raise ValueError(f"Could not find {config_field} with id {type_id}") from exc
    else:
        try:
            types = getattr(api, f"get_{tator_type}_list")(project_id)
        except Exception as exc:
            raise RuntimeError(f"Could not list {config_field}s from project {project_id}") from exc
        if list_filters:
            for attr, value in list_filters:
                types = [
                    _type
                    for _type in types
                    if hasattr(_type, attr) and getattr(_type, attr) == value
                ]
        if len(types) > 1:
            raise ValueError(
                f"Project {project_id} has more than one {config_field}, specify one of the "
                f"following in the config: {types}"
            )
        type_id = types[0].id
    return type_id


def safe_get_type(type_id: int, type_getter: Callable[[int], Any]):
    type_inst = None
    try:
        type_inst = type_getter(type_id)
    except Exception:
        logger.error("Could not find type %d", type_id, exc_info=True)
    return type_inst


def file_exists(filename: str) -> str:
    if os.path.isfile(filename):
        return filename
    raise argparse.ArgumentTypeError("Expected the name of an existing file!")


def build_section_name(
    vessel_name: str, sail_date: Optional[date] = None, land_date: Optional[date] = None
) -> str:
    try:
        sail_date_str = sail_date.strftime("%Y-%m-%d") if sail_date else SAIL_DATE_PLACEHOLDER
    except Exception:
        sail_date_str = SAIL_DATE_PLACEHOLDER
    try:
        land_date_str = land_date.strftime("%Y-%m-%d") if land_date else LAND_DATE_PLACEHOLDER
    except Exception:
        land_date_str = LAND_DATE_PLACEHOLDER

    return f"{vessel_name} ({sail_date_str} - {land_date_str})"


def section_from_name(tator_api: TatorApi, project_id: int, section_name: str) -> Optional[Section]:
    section = None
    try:
        section_list = tator_api.get_section_list(project_id)
    except Exception:
        pass
    else:
        if isinstance(section_list, list):
            for sec in section_list:
                if sec.name == section_name:
                    section = sec
                    break
    return section
